@@ -19,7 +19,7 @@ def get_include_path():
 def _compute_signature():
     # Compute signature of ufc header files
     h = hashlib.sha1()
-    with open(os.path.join(get_include_path(), "ufc.h")) as f:
+    with open(os.path.join(get_include_path(), "ufcx.h")) as f:
         h.update(f.read().encode("utf-8"))
     return h.hexdigest()
 
@@ -28,7 +28,7 @@ _signature = _compute_signature()
 
 
 def get_signature():
-    """Return SHA-1 hash of the contents of ufc.h.
+    """Return SHA-1 hash of the contents of ufcx.h.
 
     In this implementation, the value is computed on import.
     """
