@@ -9,6 +9,7 @@ import logging
 
 import ufl
 from ffcx.element_interface import create_element
+from ffcx.naming import scalar_to_value_type
 
 logger = logging.getLogger("ffcx")
 
@@ -122,12 +123,13 @@ class FFCXBackendDefinitions(object):
 
         # Get properties of domain
         domain = mt.terminal.ufl_domain()
-        gdim = domain.geometric_dimension()
         coordinate_element = domain.ufl_coordinate_element()
         num_scalar_dofs = create_element(coordinate_element).sub_element.dim
 
-        # Reference coordinates are known, no coordinate field, so we compute
-        # this component as linear combination of coordinate_dofs "dofs" and table
+        num_dofs = tabledata.values.shape[3]
+        begin = tabledata.offset
+
+        assert num_scalar_dofs == num_dofs
 
         # Find table name
         ttype = tabledata.ttype
@@ -135,15 +137,23 @@ class FFCXBackendDefinitions(object):
         assert ttype != "zeros"
         assert ttype != "ones"
 
-        begin = tabledata.offset
-        num_dofs = tabledata.values.shape[3]
-        bs = tabledata.block_size
-
-        # Inlined version (we know this is bounded by a small number)
+        # Get access to element table
         FE = self.symbols.element_table(tabledata, self.entitytype, mt.restriction)
-        dof_access = self.symbols.domain_dofs_access(gdim, num_scalar_dofs, mt.restriction)
-        value = L.Sum([dof_access[begin + i * bs] * FE[i] for i in range(num_dofs)])
-        code = [L.VariableDecl("const double", access, value)]
+        ic = self.symbols.coefficient_dof_sum_index()
+        dof_access = self.symbols.S("coordinate_dofs")
+
+        # coordinate dofs is always 3d
+        dim = 3
+        offset = 0
+        if mt.restriction == "-":
+            offset = num_scalar_dofs * dim
+
+        value_type = scalar_to_value_type(self.parameters["scalar_type"])
+
+        code = []
+        body = [L.AssignAdd(access, dof_access[ic * dim + begin + offset] * FE[ic])]
+        code += [L.VariableDecl(f"{value_type}", access, 0.0)]
+        code += [L.ForRange(ic, 0, num_scalar_dofs, body)]
 
         return [], code
 
@@ -164,17 +174,11 @@ class FFCXBackendDefinitions(object):
             if mt.local_derivatives:
                 logging.exception("FIXME: Jacobian in custom integrals is not implemented.")
             return []
-        elif self.integral_type == "expression":
-            return self._define_coordinate_dofs_lincomb(e, mt, tabledata, quadrature_rule, access)
         else:
             return self._define_coordinate_dofs_lincomb(e, mt, tabledata, quadrature_rule, access)
 
     def jacobian(self, e, mt, tabledata, quadrature_rule, access):
-        """Return definition code for the Jacobian of x(X).
-
-        J = sum_k xdof_k grad_X xphi_k(X)
-        """
-        # TODO: Jacobian may need adjustment for custom_integral_types
+        """Return definition code for the Jacobian of x(X)."""
         return self._define_coordinate_dofs_lincomb(e, mt, tabledata, quadrature_rule, access)
 
     def _expect_table(self, e, mt, tabledata, quadrature_rule, access):
