@@ -8,13 +8,16 @@
 # old implementation in FFC
 
 import logging
+import typing
 
 import ffcx.codegeneration.dofmap_template as ufcx_dofmap
 
+
 logger = logging.getLogger("ffcx")
 
 
-def tabulate_entity_dofs(L, entity_dofs, num_dofs_per_entity):
+def tabulate_entity_dofs(L, entity_dofs: typing.List[typing.List[typing.List[int]]],
+                         num_dofs_per_entity: typing.List[int]):
     # Output argument array
     dofs = L.Symbol("dofs")
 
@@ -70,7 +73,7 @@ def generator(ir, parameters):
     import ffcx.codegeneration.C.cnodes as L
 
     num_entity_dofs = ir.num_entity_dofs + [0, 0, 0, 0]
-    num_entity_dofs = num_entity_dofs[:4]
+    num_entity_dofs = num_entity_dofs[: 4]
     d["num_entity_dofs"] = f"num_entity_dofs_{ir.name}"
     d["num_entity_dofs_init"] = L.ArrayDecl("int", f"num_entity_dofs_{ir.name}",
                                             values=num_entity_dofs, sizes=4)
@@ -84,8 +87,8 @@ def generator(ir, parameters):
     d["block_size"] = ir.block_size
 
     # Functions
-    d["tabulate_entity_dofs"] = tabulate_entity_dofs(L, *ir.tabulate_entity_dofs)
-    d["tabulate_entity_closure_dofs"] = tabulate_entity_dofs(L, *ir.tabulate_entity_closure_dofs)
+    d["tabulate_entity_dofs"] = tabulate_entity_dofs(L, ir.entity_dofs, ir.num_entity_dofs)
+    d["tabulate_entity_closure_dofs"] = tabulate_entity_dofs(L, ir.entity_closure_dofs, ir.num_entity_closure_dofs)
 
     if len(ir.sub_dofmaps) > 0:
         d["sub_dofmaps_initialization"] = L.ArrayDecl(
