@@ -16,7 +16,7 @@ from ffcx.ir.representation import compute_ir
 def count_flops(form: ufl.Form, parameters: Optional[dict] = {}):
     """Return a list with the number of flops for each kernel in the Form."""
     parameters = ffcx.parameters.get_parameters(parameters)
-    assert(isinstance(form, ufl.Form))
+    assert isinstance(form, ufl.Form)
     analysis = analyze_ufl_objects([form], parameters)
     ir = compute_ir(analysis, {}, "flops", parameters, False)
 
