@@ -18,13 +18,16 @@ representation under the key "foo".
 
 import itertools
 import logging
+import numbers
+import typing
 import warnings
-from collections import namedtuple
 
+import basix
 import numpy
 import ufl
 from ffcx import naming
-from ffcx.element_interface import create_element
+from ffcx.analysis import UFLData
+from ffcx.element_interface import convert_element
 from ffcx.ir.integral import compute_integral_ir
 from ffcx.ir.representationutils import (QuadratureRule,
                                          create_quadrature_points_and_weights)
@@ -33,40 +36,137 @@ from ufl.sorting import sorted_expr_sum
 
 logger = logging.getLogger("ffcx")
 
-ir_form = namedtuple('ir_form', [
-    'id', 'name', 'signature', 'rank', 'num_coefficients', 'num_constants',
-    'name_from_uflfile', 'function_spaces', 'original_coefficient_position',
-    'coefficient_names', 'constant_names', 'finite_elements',
-    'dofmaps', 'integral_names', 'subdomain_ids'])
-ir_element = namedtuple('ir_element', [
-    'id', 'name', 'signature', 'cell_shape', 'topological_dimension',
-    'geometric_dimension', 'space_dimension', 'value_shape', 'reference_value_shape', 'degree',
-    'family', 'num_sub_elements', 'block_size', 'sub_elements', 'element_type', 'entity_dofs',
-    'lagrange_variant', 'dpc_variant', 'basix_family', 'basix_cell', 'discontinuous', 'custom_element'])
-ir_dofmap = namedtuple('ir_dofmap', [
-    'id', 'name', 'signature', 'num_global_support_dofs', 'num_element_support_dofs', 'num_entity_dofs',
-    'tabulate_entity_dofs', 'num_entity_closure_dofs', 'tabulate_entity_closure_dofs', 'num_sub_dofmaps',
-    'sub_dofmaps', 'block_size'])
-ir_integral = namedtuple('ir_integral', [
-    'integral_type', 'subdomain_id', 'rank', 'geometric_dimension', 'topological_dimension', 'entitytype',
-    'num_facets', 'num_vertices', 'enabled_coefficients', 'element_dimensions',
-    'element_ids', 'tensor_shape', 'coefficient_numbering', 'coefficient_offsets',
-    'original_constant_offsets', 'params', 'cell_shape', 'unique_tables', 'unique_table_types',
-    'table_dofmaps', 'integrand', 'name', 'precision', 'needs_facet_permutations', 'coordinate_element'])
-ir_expression = namedtuple('ir_expression', [
-    'name', 'element_dimensions', 'params', 'unique_tables', 'unique_table_types', 'integrand',
-    'table_dofmaps', 'coefficient_numbering', 'coefficient_offsets',
-    'integral_type', 'entitytype', 'tensor_shape', 'expression_shape', 'original_constant_offsets',
-    'original_coefficient_positions', 'points', 'coefficient_names', 'constant_names', 'needs_facet_permutations',
-    'function_spaces', 'name_from_uflfile'])
-ir_custom_element = namedtuple('ir_custom_element', [
-    'cell_type', 'value_shape', 'wcoeffs', 'x', 'M', 'map_type',
-    'discontinuous', 'highest_complete_degree', 'highest_degree'])
 
-ir_data = namedtuple('ir_data', ['elements', 'dofmaps', 'integrals', 'forms', 'expressions'])
+class FormIR(typing.NamedTuple):
+    id: int
+    name: str
+    signature: str
+    rank: int
+    num_coefficients: int
+    num_constants: int
+    name_from_uflfile: str
+    function_spaces: typing.Dict[str, typing.Tuple[str, str, str, int, basix.CellType, basix.LagrangeVariant]]
+    original_coefficient_position: typing.List[int]
+    coefficient_names: typing.List[str]
+    constant_names: typing.List[str]
+    finite_elements: typing.List[str]
+    dofmaps: typing.List[str]
+    integral_names: typing.Dict[str, typing.List[str]]
+    subdomain_ids: typing.Dict[str, typing.List[int]]
 
 
-def compute_ir(analysis, object_names, prefix, parameters, visualise):
+class CustomElementIR(typing.NamedTuple):
+    cell_type: basix.CellType
+    value_shape: typing.Tuple[int, ...]
+    wcoeffs: numpy.typing.NDArray[numpy.float64]
+    x: typing.List[typing.List[numpy.typing.NDArray[numpy.float64]]]
+    M: typing.List[typing.List[numpy.typing.NDArray[numpy.float64]]]
+    map_type: basix.MapType
+    interpolation_nderivs: int
+    discontinuous: bool
+    highest_complete_degree: int
+    highest_degree: int
+
+
+class ElementIR(typing.NamedTuple):
+    id: int
+    name: str
+    signature: str
+    cell_shape: str
+    topological_dimension: int
+    geometric_dimension: int
+    space_dimension: int
+    value_shape: typing.Tuple[int, ...]
+    reference_value_shape: typing.Tuple[int, ...]
+    degree: int
+    family: str
+    num_sub_elements: int
+    block_size: int
+    sub_elements: typing.List[str]
+    element_type: str
+    entity_dofs: typing.List[typing.List[typing.List[int]]]
+    lagrange_variant: basix.LagrangeVariant
+    dpc_variant: basix.DPCVariant
+    basix_family: basix.ElementFamily
+    basix_cell: basix.CellType
+    discontinuous: bool
+    custom_element: CustomElementIR
+
+
+class DofMapIR(typing.NamedTuple):
+    id: int
+    name: str
+    signature: str
+    num_global_support_dofs: int
+    num_element_support_dofs: int
+    entity_dofs: typing.List[typing.List[typing.List[int]]]
+    num_entity_dofs: typing.List[typing.List[int]]
+    entity_closure_dofs: typing.List[typing.List[typing.List[int]]]
+    num_entity_closure_dofs: typing.List[typing.List[int]]
+    num_sub_dofmaps: int
+    sub_dofmaps: typing.List[str]
+    block_size: int
+
+
+class IntegralIR(typing.NamedTuple):
+    integral_type: str
+    subdomain_id: typing.Union[str, typing.Tuple[int, ...], int]
+    rank: int
+    geometric_dimension: int
+    topological_dimension: int
+    entitytype: str
+    num_facets: int
+    num_vertices: int
+    enabled_coefficients: typing.List[bool]
+    element_dimensions: typing.Dict[ufl.FiniteElementBase, int]
+    element_ids: typing.Dict[ufl.FiniteElementBase, int]
+    tensor_shape: typing.List[int]
+    coefficient_numbering: typing.Dict[ufl.Coefficient, int]
+    coefficient_offsets: typing.Dict[ufl.Coefficient, int]
+    original_constant_offsets: typing.Dict[ufl.Constant, int]
+    params: dict
+    cell_shape: str
+    unique_tables: typing.Dict[str, numpy.typing.NDArray[numpy.float64]]
+    unique_table_types: typing.Dict[str, str]
+    integrand: typing.Dict[QuadratureRule, dict]
+    name: str
+    precision: int
+    needs_facet_permutations: bool
+    coordinate_element: str
+
+
+class ExpressionIR(typing.NamedTuple):
+    name: str
+    element_dimensions: typing.Dict[ufl.FiniteElementBase, int]
+    params: dict
+    unique_tables: typing.Dict[str, numpy.typing.NDArray[numpy.float64]]
+    unique_table_types: typing.Dict[str, str]
+    integrand: typing.Dict[QuadratureRule, dict]
+    coefficient_numbering: typing.Dict[ufl.Coefficient, int]
+    coefficient_offsets: typing.Dict[ufl.Coefficient, int]
+    integral_type: str
+    entitytype: str
+    tensor_shape: typing.List[int]
+    expression_shape: typing.List[int]
+    original_constant_offsets: typing.Dict[ufl.Constant, int]
+    points: numpy.typing.NDArray[numpy.float64]
+    coefficient_names: typing.List[str]
+    constant_names: typing.List[str]
+    needs_facet_permutations: bool
+    function_spaces: typing.Dict[str, typing.Tuple[str, str, str, int, basix.CellType, basix.LagrangeVariant]]
+    name_from_uflfile: str
+    original_coefficient_positions: typing.List[int]
+
+
+class DataIR(typing.NamedTuple):
+    elements: typing.List[ElementIR]
+    dofmaps: typing.List[DofMapIR]
+    integrals: typing.List[IntegralIR]
+    forms: typing.List[FormIR]
+    expressions: typing.List[ExpressionIR]
+
+
+def compute_ir(analysis: UFLData, object_names, prefix, parameters, visualise):
     """Compute intermediate representation."""
     logger.info(79 * "*")
     logger.info("Compiler stage 2: Computing intermediate representation of objects")
@@ -112,62 +212,59 @@ def compute_ir(analysis, object_names, prefix, parameters, visualise):
                                              finite_element_names, dofmap_names)
                       for i, expr in enumerate(analysis.expressions)]
 
-    return ir_data(elements=ir_elements, dofmaps=ir_dofmaps,
-                   integrals=ir_integrals, forms=ir_forms,
-                   expressions=ir_expressions)
+    return DataIR(elements=ir_elements, dofmaps=ir_dofmaps,
+                  integrals=ir_integrals, forms=ir_forms,
+                  expressions=ir_expressions)
 
 
-def _compute_element_ir(ufl_element, element_numbers, finite_element_names):
+def _compute_element_ir(element, element_numbers, finite_element_names):
     """Compute intermediate representation of element."""
-    logger.info(f"Computing IR for element {ufl_element}")
+    logger.info(f"Computing IR for element {element}")
+
+    element = convert_element(element)
 
     # Create basix elements
-    basix_element = create_element(ufl_element)
-    cell = ufl_element.cell()
-    cellname = cell.cellname()
+    cell = element.cell()
 
     # Store id
-    ir = {"id": element_numbers[ufl_element]}
-    ir["name"] = finite_element_names[ufl_element]
+    ir = {"id": element_numbers[element]}
+    ir["name"] = finite_element_names[element]
 
     # Compute data for each function
-    ir["signature"] = repr(ufl_element)
-    ir["cell_shape"] = cellname
+    ir["signature"] = repr(element)
+    ir["cell_shape"] = element.cell_type.name
     ir["topological_dimension"] = cell.topological_dimension()
     ir["geometric_dimension"] = cell.geometric_dimension()
-    ir["space_dimension"] = basix_element.dim
-    ir["element_type"] = basix_element.element_type
-    ir["lagrange_variant"] = basix_element.lagrange_variant
-    ir["dpc_variant"] = basix_element.dpc_variant
-    ir["basix_family"] = basix_element.element_family
-    ir["basix_cell"] = basix_element.cell_type
-    ir["discontinuous"] = basix_element.discontinuous
-    ir["degree"] = ufl_element.degree()
-    ir["family"] = ufl_element.family()
-    ir["value_shape"] = ufl_element.value_shape()
-    ir["reference_value_shape"] = ufl_element.reference_value_shape()
+    ir["space_dimension"] = element.dim
+    ir["element_type"] = element.ufcx_element_type
+    ir["lagrange_variant"] = element.lagrange_variant
+    ir["dpc_variant"] = element.dpc_variant
+    ir["basix_family"] = element.element_family
+    ir["basix_cell"] = element.cell_type
+    ir["discontinuous"] = element.discontinuous
+    ir["degree"] = element.degree()
+    ir["family"] = element.family_name
+    ir["value_shape"] = element.value_shape()
+    ir["reference_value_shape"] = element.reference_value_shape()
 
-    ir["num_sub_elements"] = ufl_element.num_sub_elements()
-    ir["sub_elements"] = [finite_element_names[e] for e in ufl_element.sub_elements()]
+    ir["num_sub_elements"] = element.num_sub_elements()
+    ir["sub_elements"] = [finite_element_names[e] for e in element.sub_elements()]
 
-    if hasattr(basix_element, "block_size"):
-        ir["block_size"] = basix_element.block_size
-        ufl_element = ufl_element.sub_elements()[0]
-        basix_element = create_element(ufl_element)
-    else:
-        ir["block_size"] = 1
+    ir["block_size"] = element.block_size
+    if element.block_size > 1:
+        element = element.sub_element
 
-    ir["entity_dofs"] = basix_element.entity_dofs
+    ir["entity_dofs"] = element.entity_dofs
 
-    if basix_element.is_custom_element:
-        ir["custom_element"] = _compute_custom_element_ir(basix_element.element)
+    if element.is_custom_element:
+        ir["custom_element"] = _compute_custom_element_ir(element.element)
     else:
         ir["custom_element"] = None
 
-    return ir_element(**ir)
+    return ElementIR(**ir)
 
 
-def _compute_custom_element_ir(basix_element):
+def _compute_custom_element_ir(basix_element: basix.finite_element.FiniteElement):
     """Compute intermediate representation of a custom Basix element."""
     ir = {}
     ir["cell_type"] = basix_element.cell_type
@@ -177,54 +274,54 @@ def _compute_custom_element_ir(basix_element):
     ir["M"] = basix_element.M
     ir["map_type"] = basix_element.map_type
     ir["discontinuous"] = basix_element.discontinuous
+    ir["interpolation_nderivs"] = basix_element.interpolation_nderivs
     ir["highest_complete_degree"] = basix_element.highest_complete_degree
     ir["highest_degree"] = basix_element.highest_degree
 
-    return ir_custom_element(**ir)
+    return CustomElementIR(**ir)
 
 
-def _compute_dofmap_ir(ufl_element, element_numbers, dofmap_names):
+def _compute_dofmap_ir(element, element_numbers, dofmap_names):
     """Compute intermediate representation of dofmap."""
-    logger.info(f"Computing IR for dofmap of {ufl_element}")
+    logger.info(f"Computing IR for dofmap of {element}")
 
     # Create basix elements
-    basix_element = create_element(ufl_element)
+    element = convert_element(element)
 
     # Store id
-    ir = {"id": element_numbers[ufl_element]}
-    ir["name"] = dofmap_names[ufl_element]
+    ir = {"id": element_numbers[element]}
+    ir["name"] = dofmap_names[element]
 
     # Compute data for each function
-    ir["signature"] = "FFCx dofmap for " + repr(ufl_element)
-    ir["sub_dofmaps"] = [dofmap_names[e] for e in ufl_element.sub_elements()]
-    ir["num_sub_dofmaps"] = ufl_element.num_sub_elements()
+    ir["signature"] = "FFCx dofmap for " + repr(element)
+    ir["sub_dofmaps"] = [dofmap_names[e] for e in element.sub_elements()]
+    ir["num_sub_dofmaps"] = element.num_sub_elements()
 
-    if hasattr(basix_element, "block_size"):
-        ir["block_size"] = basix_element.block_size
-        basix_element = basix_element.sub_element
-    else:
-        ir["block_size"] = 1
+    ir["block_size"] = element.block_size
+    if element.block_size > 1:
+        element = element.sub_element
 
     # Precompute repeatedly used items
-    for i in basix_element.num_entity_dofs:
+    for i in element.num_entity_dofs:
         # FIXME: this assumes the same number of DOFs on each entity of the same dim: this
         # assumption will not be true for prisms and pyramids
         if max(i) != min(i):
             raise RuntimeError("Elements with different numbers of DOFs on subentities of the same dimension"
                                " are not yet supported in FFCx.")
 
-    num_dofs_per_entity = [i[0] for i in basix_element.num_entity_dofs]
+    # FIXME: This does not work for prisms and pyramids
+    num_dofs_per_entity = [i[0] for i in element.num_entity_dofs]
     ir["num_entity_dofs"] = num_dofs_per_entity
-    ir["tabulate_entity_dofs"] = (basix_element.entity_dofs, num_dofs_per_entity)
+    ir["entity_dofs"] = element.entity_dofs
 
-    num_dofs_per_entity_closure = [i[0] for i in basix_element.num_entity_closure_dofs]
+    num_dofs_per_entity_closure = [i[0] for i in element.num_entity_closure_dofs]
     ir["num_entity_closure_dofs"] = num_dofs_per_entity_closure
-    ir["tabulate_entity_closure_dofs"] = (basix_element.entity_closure_dofs, num_dofs_per_entity_closure)
+    ir["entity_closure_dofs"] = element.entity_closure_dofs
 
-    ir["num_global_support_dofs"] = basix_element.num_global_support_dofs
-    ir["num_element_support_dofs"] = basix_element.dim - ir["num_global_support_dofs"]
+    ir["num_global_support_dofs"] = element.num_global_support_dofs
+    ir["num_element_support_dofs"] = element.dim - ir["num_global_support_dofs"]
 
-    return ir_dofmap(**ir)
+    return DofMapIR(**ir)
 
 
 def _compute_integral_ir(form_data, form_index, element_numbers, integral_names,
@@ -262,25 +359,22 @@ def _compute_integral_ir(form_data, form_index, element_numbers, integral_names,
             "num_vertices": cell.num_vertices(),
             "enabled_coefficients": itg_data.enabled_coefficients,
             "cell_shape": cellname,
-            "coordinate_element": finite_element_names[itg_data.domain.ufl_coordinate_element()]
+            "coordinate_element": finite_element_names[convert_element(itg_data.domain.ufl_coordinate_element())]
         }
 
         # Get element space dimensions
         unique_elements = element_numbers.keys()
-        ir["element_dimensions"] = {
-            ufl_element: create_element(ufl_element).dim
-            for ufl_element in unique_elements
-        }
+        ir["element_dimensions"] = {element: element.dim for element in unique_elements}
 
         ir["element_ids"] = {
-            ufl_element: i
-            for i, ufl_element in enumerate(unique_elements)
+            element: i
+            for i, element in enumerate(unique_elements)
         }
 
         # Create dimensions of primary indices, needed to reset the argument
         # 'A' given to tabulate_tensor() by the assembler.
         argument_dimensions = [
-            ir["element_dimensions"][ufl_element] for ufl_element in form_data.argument_elements
+            ir["element_dimensions"][convert_element(element)] for element in form_data.argument_elements
         ]
 
         # Compute shape of element tensor
@@ -364,7 +458,7 @@ def _compute_integral_ir(form_data, form_index, element_numbers, integral_names,
         _offset = 0
         for k, el in zip(index_to_coeff, form_data.coefficient_elements):
             offsets[k[1]] = _offset
-            _offset += width * ir["element_dimensions"][el]
+            _offset += width * ir["element_dimensions"][convert_element(el)]
 
         # Copy offsets also into IR
         ir["coefficient_offsets"] = offsets
@@ -377,7 +471,6 @@ def _compute_integral_ir(form_data, form_index, element_numbers, integral_names,
             _offset += numpy.product(constant.ufl_shape, dtype=int)
 
         ir["original_constant_offsets"] = original_constant_offsets
-
         ir["precision"] = itg_data.metadata["precision"]
 
         # Create map from number of quadrature points -> integrand
@@ -393,13 +486,13 @@ def _compute_integral_ir(form_data, form_index, element_numbers, integral_names,
         # Fetch name
         ir["name"] = integral_names[(form_index, itg_data_index)]
 
-        irs.append(ir_integral(**ir))
+        irs.append(IntegralIR(**ir))
 
     return irs
 
 
 def _compute_form_ir(form_data, form_id, prefix, form_names, integral_names, element_numbers, finite_element_names,
-                     dofmap_names, object_names):
+                     dofmap_names, object_names) -> FormIR:
     """Compute intermediate representation of form."""
     logger.info(f"Computing IR for form {form_id}")
 
@@ -424,36 +517,36 @@ def _compute_form_ir(form_data, form_id, prefix, form_names, integral_names, ele
     ir["original_coefficient_position"] = form_data.original_coefficient_positions
 
     ir["finite_elements"] = [
-        finite_element_names[e]
+        finite_element_names[convert_element(e)]
         for e in form_data.argument_elements + form_data.coefficient_elements
     ]
 
     ir["dofmaps"] = [
-        dofmap_names[e] for e in form_data.argument_elements + form_data.coefficient_elements
+        dofmap_names[convert_element(e)] for e in form_data.argument_elements + form_data.coefficient_elements
     ]
 
     fs = {}
     for function in form_data.original_form.arguments() + tuple(form_data.reduced_coefficients):
         name = object_names.get(id(function), str(function))
-        el = function.ufl_function_space().ufl_element()
+        el = convert_element(convert_element(function.ufl_function_space().ufl_element()))
         cmap = function.ufl_function_space().ufl_domain().ufl_coordinate_element()
         # Default point spacing for CoordinateElement is equispaced
         if cmap.variant() is None:
             cmap._sub_element._variant = "equispaced"
-        basix_cmap = create_element(cmap)
+        cmap = convert_element(cmap)
         family = cmap.family()
         degree = cmap.degree()
         fs[name] = (finite_element_names[el], dofmap_names[el], family, degree,
-                    basix_cmap.cell_type, basix_cmap.lagrange_variant)
+                    cmap.cell_type, cmap.lagrange_variant)
 
     form_name = object_names.get(id(form_data.original_form), form_id)
 
     ir["function_spaces"] = fs
     ir["name_from_uflfile"] = f"form_{prefix}_{form_name}"
 
-    # Store names of integrals and subdomain_ids for this form, grouped by integral types
-    # Since form points to all integrals it contains, it has to know their names
-    # for codegen phase
+    # Store names of integrals and subdomain_ids for this form, grouped
+    # by integral types Since form points to all integrals it contains,
+    # it has to know their names for codegen phase
     ir["integral_names"] = {}
     ir["subdomain_ids"] = {}
     ufcx_integral_types = ("cell", "exterior_facet", "interior_facet")
@@ -464,8 +557,9 @@ def _compute_form_ir(form_data, form_id, prefix, form_names, integral_names, ele
         for itg_index, itg_data in enumerate(form_data.integral_data):
             if (itg_data.integral_type == integral_type):
                 if itg_data.subdomain_id == "otherwise":
-                    # UFL is using "otherwise" for default integrals (over whole mesh)
-                    # but FFCx needs integers, so otherwise = -1
+                    # UFL is using "otherwise" for default integrals
+                    # (over whole mesh) but FFCx needs integers, so
+                    # otherwise = -1
                     if len(ir["subdomain_ids"][integral_type]) > 0 and ir["subdomain_ids"][integral_type][0] == -1:
                         raise ValueError("Only one default ('otherwise') integral allowed.")
 
@@ -476,11 +570,11 @@ def _compute_form_ir(form_data, form_id, prefix, form_names, integral_names, ele
                 elif itg_data.subdomain_id < 0:
                     raise ValueError("Integral subdomain ID must be non-negative.")
                 else:
-                    assert isinstance(itg_data.subdomain_id, int)
+                    assert isinstance(itg_data.subdomain_id, numbers.Integral)
                     ir["subdomain_ids"][integral_type] += [itg_data.subdomain_id]
                     ir["integral_names"][integral_type] += [integral_names[(form_id, itg_index)]]
 
-    return ir_form(**ir)
+    return FormIR(**ir)
 
 
 def _compute_expression_ir(expression, index, prefix, analysis, parameters, visualise, object_names,
@@ -508,17 +602,12 @@ def _compute_expression_ir(expression, index, prefix, analysis, parameters, visu
 
     # Prepare dimensions of all unique element in expression, including
     # elements for arguments, coefficients and coordinate mappings
-    ir["element_dimensions"] = {
-        ufl_element: create_element(ufl_element).dim
-        for ufl_element in analysis.unique_elements
-    }
+    ir["element_dimensions"] = {element: element.dim for element in analysis.unique_elements}
 
     # Extract dimensions for elements of arguments only
     arguments = ufl.algorithms.extract_arguments(expression)
-    argument_elements = tuple(f.ufl_function_space().ufl_element() for f in arguments)
-    argument_dimensions = [
-        ir["element_dimensions"][ufl_element] for ufl_element in argument_elements
-    ]
+    argument_elements = tuple(convert_element(f.ufl_function_space().ufl_element()) for f in arguments)
+    argument_dimensions = [ir["element_dimensions"][element] for element in argument_elements]
 
     tensor_shape = argument_dimensions
     ir["tensor_shape"] = tensor_shape
@@ -547,8 +636,8 @@ def _compute_expression_ir(expression, index, prefix, analysis, parameters, visu
     fs = {}
     for function in tuple(original_coefficients) + tuple(arguments):
         name = object_names.get(id(function), str(function))
-        el = function.ufl_function_space().ufl_element()
-        cmap = function.ufl_function_space().ufl_domain().ufl_coordinate_element()
+        el = convert_element(function.ufl_function_space().ufl_element())
+        cmap = convert_element(function.ufl_function_space().ufl_domain().ufl_coordinate_element())
         family = cmap.family()
         degree = cmap.degree()
         fs[name] = (finite_element_names[el], dofmap_names[el], family, degree)
@@ -563,13 +652,13 @@ def _compute_expression_ir(expression, index, prefix, analysis, parameters, visu
 
     ir["original_coefficient_positions"] = original_coefficient_positions
 
-    coefficient_elements = tuple(f.ufl_element() for f in coefficients)
+    coefficient_elements = tuple(convert_element(f.ufl_element()) for f in coefficients)
 
     offsets = {}
     _offset = 0
     for i, el in enumerate(coefficient_elements):
         offsets[coefficients[i]] = _offset
-        _offset += ir["element_dimensions"][el]
+        _offset += ir["element_dimensions"][convert_element(el)]
 
     # Copy offsets also into IR
     ir["coefficient_offsets"] = offsets
@@ -600,4 +689,4 @@ def _compute_expression_ir(expression, index, prefix, analysis, parameters, visu
 
     ir.update(expression_ir)
 
-    return ir_expression(**ir)
+    return ExpressionIR(**ir)
