@@ -4,13 +4,12 @@
 #
 # SPDX-License-Identifier:    LGPL-3.0-or-later
 
+import ffcx.codegeneration.jit
 import numpy as np
 import pytest
-
-import ffcx.codegeneration.jit
-from ffcx.naming import cdtype_to_numpy
-import ufl
 import sympy
+import ufl
+from ffcx.naming import cdtype_to_numpy, scalar_to_value_type
 from sympy.abc import x, y, z
 
 
@@ -52,16 +51,18 @@ def test_laplace_bilinear_form_2d(mode, expected_result, compile_args):
     kappa_value = np.array([[1.0, 2.0], [3.0, 4.0]])
     c = np.array(kappa_value.flatten(), dtype=np_type)
 
+    geom_type = scalar_to_value_type(mode)
+    np_gtype = cdtype_to_numpy(geom_type)
     coords = np.array([[0.0, 0.0, 0.0],
                        [1.0, 0.0, 0.0],
-                       [0.0, 1.0, 0.0]], dtype=np.float64)
+                       [0.0, 1.0, 0.0]], dtype=np_gtype)
 
     kernel = getattr(default_integral, f"tabulate_tensor_{np_type}")
 
     kernel(ffi.cast('{type} *'.format(type=mode), A.ctypes.data),
            ffi.cast('{type} *'.format(type=mode), w.ctypes.data),
            ffi.cast('{type} *'.format(type=mode), c.ctypes.data),
-           ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+           ffi.cast(f'{geom_type} *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     assert np.allclose(A, np.trace(kappa_value) * expected_result)
 
@@ -114,23 +115,26 @@ def test_mass_bilinear_form_2d(mode, expected_result, compile_args):
     w = np.array([], dtype=np_type)
     c = np.array([], dtype=np_type)
 
+    geom_type = scalar_to_value_type(mode)
+    np_gtype = cdtype_to_numpy(geom_type)
+
     ffi = module.ffi
     coords = np.array([[0.0, 0.0, 0.0],
                        [1.0, 0.0, 0.0],
-                       [0.0, 1.0, 0.0]], dtype=np.float64)
+                       [0.0, 1.0, 0.0]], dtype=np_gtype)
 
     kernel0 = ffi.cast(f"ufcx_tabulate_tensor_{np_type} *", getattr(form0, f"tabulate_tensor_{np_type}"))
     kernel0(ffi.cast('{type} *'.format(type=mode), A.ctypes.data),
             ffi.cast('{type} *'.format(type=mode), w.ctypes.data),
             ffi.cast('{type} *'.format(type=mode), c.ctypes.data),
-            ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+            ffi.cast(f'{geom_type} *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     b = np.zeros(3, dtype=np_type)
     kernel1 = ffi.cast(f"ufcx_tabulate_tensor_{np_type} *", getattr(form1, f"tabulate_tensor_{np_type}"))
     kernel1(ffi.cast('{type} *'.format(type=mode), b.ctypes.data),
             ffi.cast('{type} *'.format(type=mode), w.ctypes.data),
             ffi.cast('{type} *'.format(type=mode), c.ctypes.data),
-            ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+            ffi.cast(f'{geom_type} *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     assert np.allclose(A, expected_result)
     assert np.allclose(b, 1.0 / 6.0)
@@ -169,16 +173,19 @@ def test_helmholtz_form_2d(mode, expected_result, compile_args):
     w = np.array([], dtype=np_type)
     c = np.array([], dtype=np_type)
 
+    geom_type = scalar_to_value_type(mode)
+    np_gtype = cdtype_to_numpy(geom_type)
+
     ffi = module.ffi
     coords = np.array([[0.0, 0.0, 0.0],
                        [1.0, 0.0, 0.0],
-                       [0.0, 1.0, 0.0]], dtype=np.float64)
+                       [0.0, 1.0, 0.0]], dtype=np_gtype)
     kernel = getattr(form0, f"tabulate_tensor_{np_type}")
 
     kernel(ffi.cast('{type} *'.format(type=mode), A.ctypes.data),
            ffi.cast('{type} *'.format(type=mode), w.ctypes.data),
            ffi.cast('{type} *'.format(type=mode), c.ctypes.data),
-           ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+           ffi.cast(f'{geom_type} *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     assert np.allclose(A, expected_result)
 
@@ -215,17 +222,20 @@ def test_laplace_bilinear_form_3d(mode, expected_result, compile_args):
     w = np.array([], dtype=np_type)
     c = np.array([], dtype=np_type)
 
+    geom_type = scalar_to_value_type(mode)
+    np_gtype = cdtype_to_numpy(geom_type)
+
     ffi = module.ffi
     coords = np.array([0.0, 0.0, 0.0,
                        1.0, 0.0, 0.0,
                        0.0, 1.0, 0.0,
-                       0.0, 0.0, 1.0], dtype=np.float64)
+                       0.0, 0.0, 1.0], dtype=np_gtype)
 
     kernel = getattr(form0, f"tabulate_tensor_{np_type}")
     kernel(ffi.cast('{type} *'.format(type=mode), A.ctypes.data),
            ffi.cast('{type} *'.format(type=mode), w.ctypes.data),
            ffi.cast('{type} *'.format(type=mode), c.ctypes.data),
-           ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+           ffi.cast(f'{geom_type} *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     assert np.allclose(A, expected_result)
 
@@ -328,18 +338,21 @@ def test_interior_facet_integral(mode, compile_args):
     facets = np.array([0, 2], dtype=np.intc)
     perms = np.array([0, 1], dtype=np.uint8)
 
+    geom_type = scalar_to_value_type(mode)
+    np_gtype = cdtype_to_numpy(geom_type)
+
     coords = np.array([[0.0, 0.0, 0.0,
                         1.0, 0.0, 0.0,
                         0.0, 1.0, 0.0],
                        [1.0, 0.0, 0.0,
                        0.0, 1.0, 0.0,
-                       1.0, 1.0, 0.0]], dtype=np.float64)
+                       1.0, 1.0, 0.0]], dtype=np_gtype)
 
     kernel = getattr(integral0, f"tabulate_tensor_{np_type}")
     kernel(ffi.cast(f'{mode}  *', A.ctypes.data),
            ffi.cast(f'{mode}  *', w.ctypes.data),
            ffi.cast(f'{mode}  *', c.ctypes.data),
-           ffi.cast('double *', coords.ctypes.data), ffi.cast('int *', facets.ctypes.data),
+           ffi.cast(f'{geom_type} *', coords.ctypes.data), ffi.cast('int *', facets.ctypes.data),
            ffi.cast('uint8_t *', perms.ctypes.data))
 
 
@@ -373,15 +386,18 @@ def test_conditional(mode, compile_args):
     w1 = np.array([1.0, 1.0, 1.0], dtype=np_type)
     c = np.array([], dtype=np.float64)
 
+    geom_type = scalar_to_value_type(mode)
+    np_gtype = cdtype_to_numpy(geom_type)
+
     coords = np.array([[0.0, 0.0, 0.0],
                        [1.0, 0.0, 0.0],
-                       [0.0, 1.0, 0.0]], dtype=np.float64)
+                       [0.0, 1.0, 0.0]], dtype=np_gtype)
 
     kernel0 = ffi.cast(f"ufcx_tabulate_tensor_{np_type} *", getattr(form0, f"tabulate_tensor_{np_type}"))
     kernel0(ffi.cast('{type} *'.format(type=mode), A1.ctypes.data),
             ffi.cast('{type} *'.format(type=mode), w1.ctypes.data),
             ffi.cast('{type} *'.format(type=mode), c.ctypes.data),
-            ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+            ffi.cast(f'{geom_type} *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     expected_result = np.array([[2, -1, -1], [-1, 1, 0], [-1, 0, 1]], dtype=np_type)
     assert np.allclose(A1, expected_result)
@@ -393,7 +409,7 @@ def test_conditional(mode, compile_args):
     kernel1(ffi.cast('{type} *'.format(type=mode), A2.ctypes.data),
             ffi.cast('{type} *'.format(type=mode), w2.ctypes.data),
             ffi.cast('{type} *'.format(type=mode), c.ctypes.data),
-            ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+            ffi.cast(f'{geom_type} *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     expected_result = np.ones(3, dtype=np_type)
     assert np.allclose(A2, expected_result)
@@ -510,15 +526,18 @@ def test_lagrange_triangle(compile_args, order, mode, sym_fun, ufl_fun):
     b = np.zeros((order + 2) * (order + 1) // 2, dtype=np_type)
     w = np.array([], dtype=np_type)
 
+    geom_type = scalar_to_value_type(mode)
+    np_gtype = cdtype_to_numpy(geom_type)
+
     coords = np.array([[1.0, 0.0, 0.0],
                        [2.0, 0.0, 0.0],
-                       [0.0, 1.0, 0.0]], dtype=np.float64)
+                       [0.0, 1.0, 0.0]], dtype=np_gtype)
 
     kernel = getattr(default_integral, f"tabulate_tensor_{np_type}")
     kernel(ffi.cast('{type} *'.format(type=mode), b.ctypes.data),
            ffi.cast('{type} *'.format(type=mode), w.ctypes.data),
            ffi.NULL,
-           ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+           ffi.cast(f'{geom_type} *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     # Check that the result is the same as for sympy
     assert np.allclose(b, [float(i) for i in sym])
@@ -600,16 +619,19 @@ def test_lagrange_tetrahedron(compile_args, order, mode, sym_fun, ufl_fun):
     b = np.zeros((order + 3) * (order + 2) * (order + 1) // 6, dtype=np_type)
     w = np.array([], dtype=np_type)
 
+    geom_type = scalar_to_value_type(mode)
+    np_gtype = cdtype_to_numpy(geom_type)
+
     coords = np.array([1.0, 0.0, 0.0,
                        2.0, 0.0, 0.0,
                        0.0, 1.0, 0.0,
-                       0.0, 0.0, 1.0], dtype=np.float64)
+                       0.0, 0.0, 1.0], dtype=np_gtype)
 
     kernel = getattr(default_integral, f"tabulate_tensor_{np_type}")
     kernel(ffi.cast('{type} *'.format(type=mode), b.ctypes.data),
            ffi.cast('{type} *'.format(type=mode), w.ctypes.data),
            ffi.NULL,
-           ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+           ffi.cast(f'{geom_type} *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     # Check that the result is the same as for sympy
     assert np.allclose(b, [float(i) for i in sym])
@@ -645,3 +667,56 @@ def test_prism(compile_args):
            ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     assert np.isclose(sum(b), 0.5)
+
+
+def test_complex_operations(compile_args):
+    mode = "double _Complex"
+    cell = ufl.triangle
+    c_element = ufl.VectorElement("Lagrange", cell, 1)
+    mesh = ufl.Mesh(c_element)
+    element = ufl.VectorElement("DG", cell, 0)
+    V = ufl.FunctionSpace(mesh, element)
+    u = ufl.Coefficient(V)
+    J1 = ufl.real(u)[0] * ufl.imag(u)[1] * ufl.conj(u)[0] * ufl.dx
+    J2 = ufl.real(u[0]) * ufl.imag(u[1]) * ufl.conj(u[0]) * ufl.dx
+    forms = [J1, J2]
+
+    compiled_forms, module, code = ffcx.codegeneration.jit.compile_forms(
+        forms, parameters={'scalar_type': mode}, cffi_extra_compile_args=compile_args)
+
+    form0 = compiled_forms[0].integrals(module.lib.cell)[0]
+    form1 = compiled_forms[1].integrals(module.lib.cell)[0]
+
+    ffi = module.ffi
+    np_type = cdtype_to_numpy(mode)
+    w1 = np.array([3 + 5j, 8 - 7j], dtype=np_type)
+    c = np.array([], dtype=np_type)
+
+    geom_type = scalar_to_value_type(mode)
+    np_gtype = cdtype_to_numpy(geom_type)
+
+    coords = np.array([[0.0, 0.0, 0.0],
+                       [1.0, 0.0, 0.0],
+                       [0.0, 1.0, 0.0]], dtype=np_gtype)
+    J_1 = np.zeros((1), dtype=np_type)
+    kernel0 = ffi.cast(f"ufcx_tabulate_tensor_{np_type} *", getattr(form0, f"tabulate_tensor_{np_type}"))
+    kernel0(ffi.cast('{type} *'.format(type=mode), J_1.ctypes.data),
+            ffi.cast('{type} *'.format(type=mode), w1.ctypes.data),
+            ffi.cast('{type} *'.format(type=mode), c.ctypes.data),
+            ffi.cast(f'{geom_type} *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+
+    expected_result = np.array([0.5 * np.real(w1[0]) * np.imag(w1[1])
+                               * (np.real(w1[0]) - 1j * np.imag(w1[0]))], dtype=np_type)
+    assert np.allclose(J_1, expected_result)
+
+    J_2 = np.zeros((1), dtype=np_type)
+
+    kernel1 = ffi.cast(f"ufcx_tabulate_tensor_{np_type} *", getattr(form1, f"tabulate_tensor_{np_type}"))
+    kernel1(ffi.cast('{type} *'.format(type=mode), J_2.ctypes.data),
+            ffi.cast('{type} *'.format(type=mode), w1.ctypes.data),
+            ffi.cast('{type} *'.format(type=mode), c.ctypes.data),
+            ffi.cast(f'{geom_type} *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+
+    assert np.allclose(J_2, expected_result)
+
+    assert np.allclose(J_1, J_2)
